/* $Id: OdinCmd.h for Odin
 * $auth: Steve Torchinsky
 * $created: Calgary, Saturday, 2001 May 12, 22:25:29
 *
 * This is for the cpp version of mkbckseq
 * mkbckseq is a Bash script employing sed and gawk which
 * uses templates to create backend command scripts
 *
 * The cpp version will do the same job, and will also read
 * the mode configurations from text files exported from the
 * Radiometer_modes.xls excel spreadsheet by Urban Frisk
 *
 * OdinCmd.h defines some macros for use in the classes aosCmd, acCmd, mkbckseq
 * and also the OdinCmd class which standardises some formatting for
 * writing out the commands.
 *
 * Calgary, Monday, 2001 November 12, 06:39:04
 * minor version incremented with change to AC1 default modes
 * the version prior to this change was 2.2.6
 *
 * modes which contain the new AC1 default modes will have minor version 3
 * 
 * Calgary, Wednesday, 2001 December 05, 22:43:31
 * no more forking
 * minor version is bumped up to 4 today
 * now we read the correlator modes from the spreadsheet
 *
 * Calgary, Tuesday, 2002 February 26, 14:01:00
 *  version change from v2.4.4 to v2.5.0
 *  v2.4.4 was never released.  All changes noted in source files for v2.4.4
 *  apply to v2.5.0
 *  v2.5.0 released Tue, 26 Feb 2002 16:37:18 -0700 email to UFr and GF
 *
 * Calgary, Tuesday, 2002 February 26, 16:49:03
 *  version under development is v2.5.1
 *
 * St-Bruno, Thursday, 2002 October 10, 08:54:53
 *  v2.6.0  
 *   implement chopper mask during Position Switching
 *   we want the backends to stop taking data while the chopper
 *   moves to the calibration source.
 *
 *  add Observing Strategy macros
 */

#ifndef ODINCMD_H
#define ODINCMD_H

/* define DEBUG macro for debugging output statements */
//#define DEBUG

#ifdef MSDOS
#define SLASH "\\"
#define EXAMPLE_TOPDIR "C:\\"
#define EXAMPLE_DIR "C:\\Odin\\scripts"
#define EXAMPLE_PROMPT "> "
#else
#define SLASH "/"
#define EXAMPLE_TOPDIR "~"
#define EXAMPLE_DIR "~/Odin/scripts"
#define EXAMPLE_PROMPT "$ "
#endif

#define MKBCKSEQ_VERSION_MAJOR 2
// #define MKBCKSEQ_VERSION_MINOR 2 /* old AC1 default modes */
// #define MKBCKSEQ_VERSION_MINOR 3 /* new AC1 default modes */
// #define MKBCKSEQ_VERSION_TINY 9 /* old AC1 default modes */
// #define MKBCKSEQ_VERSION_TINY 3 /* new AC1 default modes */
#define MKBCKSEQ_VERSION_MINOR 6
#define MKBCKSEQ_VERSION_TINY 0

/* string length for a user command */
#define CMDLEN 20

/* disciplines */
#define ASTRO 1
#define STRAT 2

/* observing strategies */
#define DICKE_SWITCHING 0
#define POSITION_SWITCHING 1
#define F119_SWITCHING 2

/* ACDC */
#define ACDC1_USR 0xC
#define ACDC1 ACDC1_USR
#define ACDC2_USR 0xD
#define ACDC2 ACDC2_USR
#define NOACDC 0

/* pointing masks used for different situations */
#define POINTING_ASTRO_SUBMM 0x79
#define POINTING_ASTRO_119 0x7A
#define POINTING_STRAT 0x78

/* mechanism mask for chop synchronisation */
#define CHOP_SYNC 0x81
#define F119_SYNC 0x80

/* mask selections */
#define MASK_ACDC1 1
#define MASK_ACDC2 2
#define MASK_CHOPA 4
#define MASK_CHOPB 8
#define MASK_119  0x10

/* mech controllers */
#define MECHA_USR 4
#define MECHA MECHA_USR
#define MECHB_USR 9
#define MECHB MECHB_USR

/* chopper address */
#define MECH_CHOP 0x14
#define MECH_CHOP_STOP 0x5301
#define MECH_CHOP_RUN 0x5B00

/* Off the Shelf */
#define OTS_USR 0xE
#define OTS_MECHA 0x4C
#define OTS_MECHB 0x4E

/* chopper positions using OtS */
#define POSITION_A 1
#define POSITION_B 2

/* in the Radiometer Modes excel spreadsheet, on/off is defined as 1/0 */
#define ON 1
#define OFF 0

/* OdinCmd class, mainly for formatting the ocom command */
class OdinCmd {
 public:
  OdinCmd();
  ~OdinCmd();

  char *cmd(int, int, int, char *);
};

#endif

