/* $Id: acCmd.h for Odin
 * $auth: Steve Torchinsky
 * $created: Calgary, Tuesday, 2001 May 15, 14:25:05
 *
 * This is the part of the cpp version of mkbckseq
 * mkbckseq is a Bash script employing sed and gawk which
 * uses templates to create backend command scripts
 *
 * The cpp version will do the same job, and will also read
 * the mode configurations from text files exported from the
 * Radiometer_modes.xls excel spreadsheet by Urban Frisk
 *
 * 
 * The class template for correlator commanding is defined here.
 * 
 * class acCmd defines the SSB oscillator settings for each mode.
 * default settings are assigned upon construction, which can be
 * overridden by reading the backends.csv file exported from the
 * Radiometer Modes excel spreadsheet.
 *
 * changes:
 *
 * Calgary, Tuesday, 2002 February 05, 13:44:06
 *  v2.4.4
 *  adding macros for attenuator setting command address
 *  adding method for attenuator setting command
 *
 * Calgary, Friday, 2002 February 15, 14:49:56
 *  removing all arrays that use modeIndx.
 *  there's no reason to keep all default modes in memory
 *  removing N_ACMODES:  there is no maximum for the number of modes
 *
 * Calgary, Wednesday, 2002 February 20, 10:21:33
 *  adding macro definitions for attenuation levels:  MIN,MAX,NOMINAL
 *
 * Calgary, Monday, 2002 February 25, 15:02:31
 *  adding a macro definition for filterbank
 */


#include "OdinCmd.h"
#include "../OdinTM.h"

#ifndef ACCMD_H
#define ACCMD_H

/* AC1 is user 8 and AC2 is user 0xB */
#define AC1USR 8
#define AC2USR 0xB
#define AC1 AC1USR
#define AC2 AC2USR
#define FB 0xFB //an identity for switch statements

/* maximum number of correlator configurations is 8, so far     */
/* Calgary, Wednesday, 2001 December 05, 22:08:35:  now it's 13 */
/* Calgary, Friday, 2002 February 15, 14:52:57:  no more limit */
/* #define N_ACMODES 100 */

#define AC_CONFIG 1
#define AC_CONFIG_DAT 0

#define AC_BOOT 2
#define AC_BOOT_DAT 0xA0A0

#define AC_CALIBRATE 0x4
#define AC_CALIBRATE_DAT 0

#define AC_HALT 6
#define AC_HALT_DAT 0

#define AC_START 7
#define AC_START_DAT 0

#define AC_MASKACDC1 0x8
#define AC_MASKACDC2 0x8
#define AC_MASKCHOPA 0x9
#define AC_MASKCHOPB 0x9
#define AC_MASK119 0xA

#define AC_MODE 0xB
#define AC_INTTIME 0XB

#define AC_MODE_8 0x7F
#define AC_MODE_4 0x2A
#define AC_MODE_2 0x08
#define AC_MODE_1 0x00


#define AC_INPUT 0xC
#define AC_SWITCH 0xC
#define AC_FILTERBANK 0xC

#define AC_SWITCH_ON 1
#define AC_SWITCH_OFF 0

#define AC_FILTER_ON 0x8000
#define AC_FILTER_OFF 0

#define AC_ATT1 0x0D
#define AC_ATT2 0x0E
#define AC_ATT3 0x0F
#define AC_ATT4 0x10

#define AC_SSB4 0x11
#define AC_SSB3 0x12
#define AC_SSB2 0x13
#define AC_SSB1 0x14

#define ACID 0x15

#define AC_NSIG 0x16
#define AC_NREF 0x16
#define AC_SIGREF 0x16

#define AC_PRESCALE 0x19

/* attenuator settings */
#define AC_MIN_ATT 145
#define AC_MAX_ATT 95
#define AC_NOM_ATT 138


class acCmd {
 
 public:
  acCmd(unsigned short);
  ~acCmd();

  bool setModeError(unsigned short,unsigned short, unsigned short);
  bool setMode(unsigned short, unsigned short, unsigned short, unsigned short, unsigned short, unsigned short, char *); 

  bool selectMode(unsigned short);
  bool selectInputChannel(unsigned short);
  bool selectInputPort(unsigned short);
  unsigned short getInputChannel();
  unsigned short getInputPort();
  bool enableSwitching();
  bool disableSwitching();
  bool enableFilterbank();
  bool disableFilterbank();
  bool setIntTime(unsigned short);
  bool setMask(unsigned short, unsigned short);
  bool setPrescaler(unsigned short);
  bool setNsig(unsigned short);
  bool setNref(unsigned short);

  bool setAtt(short, unsigned short);

  char *bootCmd();
  char *configCmd();
  char *haltCmd();
  char *startCmd();
  char *calibrateCmd();
  char *idCmd();
  char *attCmd(short);
  char *attenuatorsCmd();
  char *ssbCmd();
  char *inputCmd();
  char *switchingCmd();
  char *filterCmd();
  char *modeCmd();
  char *intTimeCmd();
  char *maskCmd(unsigned short);
  char *clearMasksCmd();
  char *sigRefCmd();
  char *prescalerCmd();

  char *configurationCmd();
  

 private:
  bool defaultSettings();
  bool defaultSettingsAC1();
  bool defaultSettingsAC2();
  bool clearMasks();
  char *cmd(unsigned short,unsigned short,char *);

  unsigned short ssbFreq[4];
  char *modeDescription;
  unsigned short mux;
  unsigned short nbands();
  unsigned short mode;
  unsigned short whichAC;
  unsigned short ACn;
  unsigned short usr;
  unsigned short iport;
  unsigned short channel;
  unsigned short switching;
  unsigned short filterbank;
  unsigned short intTime;
  unsigned short pointingMask;
  unsigned short chopMask;
  unsigned short F119Mask;
  unsigned short prescaler;
  unsigned short nsig;
  unsigned short nref;
  unsigned short att[4];

};

#endif
