/* $Id: bckCmd.h for Odin
 * $auth: Steve Torchinsky
 * $created: Calgary, Wednesday, 2001 May 16, 20:47:56
 *
 * This is the part of the cpp version of mkbckseq
 * mkbckseq is a Bash script employing sed and gawk which
 * uses templates to create backend command scripts
 *
 * The cpp version will do the same job, and will also read
 * the mode configurations from text files exported from the
 * Radiometer_modes.xls excel spreadsheet by Urban Frisk
 *
 * 
 * The class template for the complete backend script commanding
 * is defined here.
 * 
 * class bckCmd defines the backend sequence header and overall
 * structure, using classes aosCmd and acCmd to construct the commands.
 *
 * default settings are assigned upon construction, which can be
 * overridden by reading the backends.csv file exported from the
 * Radiometer Modes excel spreadsheet (not yet implemented)
 *
 * changes:
 * Calgary, Thursday, 2001 June 14, 20:41:39
 *  added 119 Frequency Switching mask and assign method
 *
 * Calgary, Thursday, 2001 July 26, 08:42:51
 *  added method ocomFinalCmd()
 *  for extra commands for aero modes which are inserted from a text file
 *  (see mail from GF, Thu, 26 Jul 2001 11:06:14 +0200)
 *
 * Calgary, Friday, 2001 August 03, 10:08:24
 *  modified ocomFinalCmd() and moved file opening and reading part
 *  to class smrModes
 *
 * Calgary, Monday, 2002 February 04, 11:08:24
 *  v2.4.4
 *  self calibration section moved into its own method in preparation for 
 *  the new Radiometer_modes file format which will included self-cal=yes/no
 *
 * Calgary, Wednesday, 2002 February 20, 15:21:12
 *  renamed methods selfCalibrationCmd() etc to inputLevelCmd() etc
 *
 * Calgary, Monday, 2002 February 25, 16:05:02
 *  methods for assigning self cal modified to also assign attenuator settings
 *
 * Bordeaux-to-Paris, Sunday, 2002 October 20, 19:09:05
 *  v2.6.0
 *  adding observingStrategy
 */


#include "../OdinTM.h"
#include "OdinCmd.h"
#include "aosCmd.h"
#include "acCmd.h"


#ifndef BCKCMD_H
#define BCKCMD_H

/* the sequences will have a version number read from a file
 * as in the bash version of mkbckseq
 * version 68 was the latest one done with the bash version of mkseqbck
 */
#define SEQUENCE_VERSION_MINIMUM 69

class bckCmd {

 public:
  bckCmd();
  ~bckCmd();

  bool assignModeName(char *);
  bool assignModeVariation(char);
  bool assignMechController(char);
  bool assignSeqVersion(short);
  bool assignDiscipline(unsigned short);
  bool assignObservingStrategy(unsigned short);
  bool assignIntTime(unsigned short);
  bool assignChopMask(unsigned short);
  bool assignF119Mask(unsigned short);
  bool assignDescription(char *);
  bool assignFinalCmd(char *);

  bool enableAOS();
  bool disableAOS();
  bool enableAC1();
  bool disableAC1();
  bool enableAC2();
  bool disableAC2();
  bool enableFilterbank();
  bool disableFilterbank();

  bool enablePointing1();
  bool disablePointing1();
  bool enablePointing2();
  bool disablePointing2();

  bool assignSelfCalAOS(unsigned short,unsigned short);
  bool assignSelfCalAC1(unsigned short,
			unsigned short,
			unsigned short,
			unsigned short,
			unsigned short);
  bool assignSelfCalAC2(unsigned short,
			unsigned short,
			unsigned short,
			unsigned short,
			unsigned short);

  aosCmd *aos;
  acCmd *ac1;
  acCmd *ac2;

  char *procedureName();
  char *macroName();

  char *ocomSequence();

 private:
  char *stopChopperCmd();
  char *runChopperCmd();
  char *positionChopperCmd(unsigned short);

  char *ocomHeader();
  char *ocomFooter();

  char *aosStartCmd();
  char *acStartCmd(unsigned short);
  char *ac1StartCmd();
  char *ac2StartCmd();

  char *inputLevelUpperCmd();
  char *inputLevelLowerCmd();
  char *inputLevelCmd();

  short seqVersion;
  char *modeName;
  char modeVariation;
  unsigned short discipline;
  char *description;
  char *modeDescription();

  unsigned short usingAOS;
  unsigned short usingAC1;
  unsigned short usingAC2;

  unsigned short selfCalAOS;
  unsigned short selfCalAC1;
  unsigned short selfCalAC2;

  unsigned short pointingMask(unsigned short, unsigned short);
  unsigned short usingPointing1;
  unsigned short usingPointing2;

  char mechController;
  unsigned short intTime;
  unsigned short obsStrategy;
  unsigned short chopMask;
  unsigned short F119Mask;

  char *finalCmd;
};

#endif
