/* $Id: bck_main.cpp for Odin
 * $auth: Steve Torchinsky
 * $created: Calgary, Monday, 2001 June 04, 20:47:21
 *
 * This is the cpp version of mkbckseq
 * mkbckseq is a Bash script employing sed and gawk which
 * uses templates to create backend command scripts
 *
 * The cpp version will do the same job, and will also read
 * the mode configurations from text files exported from the
 * Radiometer_modes.xls excel spreadsheet by Urban Frisk
 *
 * this is the main program (finally!)
 *
 * changes:
 *  Calgary, Tuesday, 2001 June 19, 11:38:48
 *  name change from mkbckseq_main.cpp to bck_main.cpp
 *  (8.3 filename for convenience)
 * 
 *  Calgary, Tuesday, 2001 June 19, 22:39:45
 *    started adding help option
 */

#include <cstdlib>
#include <cstring>
#include "mkBckSeq.h"


int usage(char *fname){
  cout << "\nBackend Sequence Generator:  mkbckseq v"
       << MKBCKSEQ_VERSION_MAJOR
       << "."
       << MKBCKSEQ_VERSION_MINOR
       << "."
       <<  MKBCKSEQ_VERSION_TINY
       << ", " << __DATE__ << " " << __TIME__
       << "\nSteve Torchinsky, sat@iras.ucalgary.ca"
       << "\nfor detailed help type:  "  
       << fname
       << " -h"
       << endl;
  return 0;
}

int help(char *fname){
  cout << "\nBackend Sequence Generator:  mkbckseq v"
       << MKBCKSEQ_VERSION_MAJOR
       << "."
       << MKBCKSEQ_VERSION_MINOR
       << "."
       <<  MKBCKSEQ_VERSION_TINY
       << ", " << __DATE__ << " " << __TIME__
       << "\nSteve Torchinsky, sat@iras.ucalgary.ca"
       << "\n\nBackend Sequence Generator Detailed Instructions"
       << "\n------------------------------------------------"

	<< "\nBEFORE RUNNING:"
	<< "\n"
	<< "\nTo run this program you need the mode definition file"
	<< "\nRadiometer_modes_vNN.xls (where NN is a number)"
	<< "\n"
	<< "\nLoad the Excel spreadsheet into Microsoft Excel and select \"obs mod\""
	<< "\nas the active worksheet"
	<< "\n"
	<< "\nThe worksheet must be exported to format \"Comma Separated Values\""
	<< "\n(CSV) or to format \"Semi-Colon Separated\" Values (SKV).  To do this,"
	<< "\ngo to File->Save As, and select \"*.csv\" or \"*.skv\" from the drop down"
	<< "\nbox. The \"save as\" box should now contain the text"
	<< "\n\"Radiometer_modes_vNN.csv\" or \"Radiometer_modes_vNN.skv\" (where NN is"
	<< "\na number).  You'll get a warning message about not being able to"
	<< "\nexport the whole Excel document. Click \"ok\" to save the active"
	<< "\nworksheet (which should be \"obs mod\")."
	<< "\n"
	<< "\nFor systems which do not support long filenames, the file"
	<< "\nRadiometer_modes_vNN.csv can instead be named modesNN.csv (or"
	<< "\nmodesNN.skv).  This file should be saved in the location where the"
	<< "\nscripts will be generated."
	<< "\n"
	<< "\n"
	<< "\nEXTRA COMMANDS"
	<< "\n"
	<< "\nFor the aeronomy mode settings, in order to prevent interference with"
	<< "\npreset macro read-outs, that act on the chopper and calibration"
	<< "\nmirrors, the front-end scripts halt the readout and the back-end"
	<< "\nscripts start the read-out.  The back-end script generator adds, to"
	<< "\nthe end of the aeronomy scripts, the text of the file \"strat.txt\"."
	<< "\nThere's no syntax checking on that file, so whatever is written in"
	<< "\nthere will be added, even if it's an invalid command.  The file"
	<< "\n\"strat.txt\" should be placed in the directory where the scripts are"
	<< "\ngenerated (same as modesNN.csv)."
	<< "\n"
	<< "\nFor symmetry, there is also included the possibility to add files for astro"
	<< "\nmodes.  Create a text file astro.txt, and it will work the same."
	<< "\n"
	<< "\nIf the \"strat.txt\" or \"astro.txt\" files are not found, the scripts"
	<< "\nwill be generated without them."
	<< "\n"
	<< "\n"
	<< "\nRUN THE PROGRAM"
	<< "\n"
	<< "\nopen an msdos command window. (Start->Run->command.com) in the command"
	<< "\nwindow, make the scripts directory the current directory:"

       << "\n\n" << EXAMPLE_TOPDIR << EXAMPLE_PROMPT 
       << "cd " << EXAMPLE_DIR

       << "\n\nand then execute the program"

       << "\n\n" << EXAMPLE_DIR << EXAMPLE_PROMPT << fname

	<< "\n"
	<< "\nThe result should be three subdirectories filled with backend"
	<< "\nscripts.  The three directories correspond to:"
	<< "\n           pointing with ACDC1"
	<< "\n           pointing with ACDC2"
	<< "\n           no pointing"
	<< "\n"
	<< "\nWithin each of these subdirectories there are scripts for"
	<< "\nthe Mech A controller and the Mech B controller."

       << endl;
  return 0;
}

int main(int argc, char *argv[]){

  if (argc>1)
    if ((strcmp(argv[1],"-h")==0)
	|| (strcmp(argv[1],"--help")==0))
      return help(argv[0]);

  usage(argv[0]);
  mkBckSeq *bckseqs=new mkBckSeq;

  if (!bckseqs->mkAllSequences()) {
    cout << "\nERROR:  sequences not successfully generated\n";
    return 1;
  }

  return 0;

}
