/* $Id: smrModes.cpp for Odin
 * $auth: Steve Torchinsky
 * $created: Calgary, Sunday, 2001 May 20, 21:12:14
 *
 * This is the part of the cpp version of mkbckseq
 * mkbckseq is a Bash script employing sed and gawk which
 * uses templates to create backend command scripts
 *
 * The cpp version will do the same job, and will also read
 * the mode configurations from text files exported from the
 * Radiometer_modes.xls excel spreadsheet by Urban Frisk
 *
 * 
 * methods for class smrModes are defined here
 * 
 * the naming convention in this class is the following:
 *    names with all lower case are private variables
 *    names with mixed upper and lower case are methods
 *
 * modifications:
 * Calgary-to-Toronto, Monday, 2001 June 11, 12:31:51
 *    modes are now properly counted
 *
 * Calgary, Thursday, 2002 January 31, 08:07:44
 *    v2.4.3
 *    modes beginning with 'p' are astro modes using position switching
 *    see mail from Gunnar today
 *
 * Calgary, Monday, 2002 February 04, 15:39:03
 *    v2.4.4
 *    beginning to add functionality for manual attenuator settings.
 *    I don't have the Radiometer_modes file format yet...
 *
 * Calgary, Friday, 2002 February 22, 14:49:50
 *    removing "read description" from method readFirstLine
 *    description is read as part of the AC1 when it reads the SSB's
 *    this has been implemented for some time, so reading the description
 *    in readFirstLine was redundant
 *
 * Calgary, Friday, 2002 February 22, 14:52:39
 *    new method readSSB created from the relevant lines in method readLine
 *
 * Calgary, Monday, 2002 February 25, 10:41:29
 *    new method readAtt to read selfcal and attenuator settings
 *
 * Calgary, Monday, 2002 February 25, 12:43:21
 *    adding check for manual attenuator settings fileformat
 *    this is a place holder until I get the new format from Urban.
 *
 * Calgary, Monday, 2002 February 25, 14:56:00
 *    replacing aosflag and filterflag with whichBck to ID backends in 
 *    methods readLine, readAtt, readSSB
 *
 * Calgary, Monday, 2002 February 25, 15:32:38
 *    new method readAttErr
 *
 * Calgary, Monday, 2002 February 25, 22:37:15
 *    new method readAOSatt to get past the mode description
 *
 * St-Bruno, Thursday, 2002 October 10, 09:05:38
 *    beginning to add functionality for Observing Strategy
 *    added obs_strategy
 *
 * Bordeaux-to-Paris, Sunday, 2002 October 20, 19:20:05
 *    added method ObservingStrategy
 */

#include <cstdlib>
#include <fstream>
#include <cstring>
#include <strstream>
#include <iomanip>

#include "smrModes.h"
#include "acCmd.h"
#include "aosCmd.h"

smrModes::smrModes(){
  delimiter=new char[2];
  *delimiter=',';
  *(delimiter+1)='\0';
}

smrModes::~smrModes(){
}

/* public method, return the number of modes */
short smrModes::nModes(){
  return nmodes;
}

/* public method, return the mode name for a given mode */
char *smrModes::modeName(unsigned short n){
  return *(name+n);
}

/* public method, return the discipline for a given mode */
unsigned short smrModes::Discipline(unsigned short n){
  return *(discipline+n);
}

/* public method, return the observing strategy for a given mode */
unsigned short smrModes::ObservingStrategy(unsigned short n){
  return *(obs_strategy+n);
}

/* public method, return the mode variation for a given mode */
char smrModes::modeVariation(unsigned short n){
  return *(variation+n);
}

/* public method, return the mode description for a given mode */
char *smrModes::modeDescription(unsigned short n){
  return *(description+n);
}

/* public method, return the output mode of the aos */
unsigned short smrModes::aosMode(unsigned short n){
  return *(aosmode+n);
}

/* public method, return the status of the aos for a given mode */
unsigned short smrModes::usingAOS(unsigned short n){
  return *(usingaos+n);
}

/* public method, return the input port of the aos for a given mode */
unsigned short smrModes::aosInput(unsigned short n){
  return *(aosinput+n);
}

/* public method, return the input cal status AOS */
unsigned short smrModes::aosSelfCal(unsigned short n){
  return *(selfcalaos+n);
}

/* public method, return attenuator setting for AOS */
unsigned short smrModes::aosATT(unsigned short n){
  return *(aosatt+n);
}


/* public method, return the mode of the ac1 */
unsigned short smrModes::ac1Mode(unsigned short n){
  return *(ac1mode+n);
}

/* public method, return the status of the ac1 for a given mode */
unsigned short smrModes::usingAC1(unsigned short n){
  return *(usingac1+n);
}

/* public method, return the input port of the ac1 for a given mode */
unsigned short smrModes::ac1Input(unsigned short n){
  return *(ac1input+n);
}

/* public method, return the mux setting for AC1 */
unsigned short smrModes::ac1Mux(unsigned short n){
  return *(ac1mux+n);
}

/* public method, return the SSB1 setting for AC1 */
unsigned short smrModes::ac1SSB1(unsigned short n){
  return *(ac1ssb1+n);
}

/* public method, return the SSB2 setting for AC1 */
unsigned short smrModes::ac1SSB2(unsigned short n){
  return *(ac1ssb2+n);
}

/* public method, return the SSB3 setting for AC1 */
unsigned short smrModes::ac1SSB3(unsigned short n){
  return *(ac1ssb3+n);
}

/* public method, return the SSB4 setting for AC1 */
unsigned short smrModes::ac1SSB4(unsigned short n){
  return *(ac1ssb4+n);
}

/* public method, return the input cal status AC1 */
unsigned short smrModes::ac1SelfCal(unsigned short n){
  return *(selfcalac1+n);
}

/* public method, return attenuator setting 1 for AC1 */
unsigned short smrModes::ac1ATT1(unsigned short n){
  return *(ac1att1+n);
}

/* public method, return attenuator setting 2 for AC1 */
unsigned short smrModes::ac1ATT2(unsigned short n){
  return *(ac1att2+n);
}

/* public method, return attenuator setting 3 for AC1 */
unsigned short smrModes::ac1ATT3(unsigned short n){
  return *(ac1att3+n);
}

/* public method, return attenuator setting 4 for AC1 */
unsigned short smrModes::ac1ATT4(unsigned short n){
  return *(ac1att4+n);
}

/* public method, return the mode of the ac2 */
unsigned short smrModes::ac2Mode(unsigned short n){
  return *(ac2mode+n);
}

/* public method, return the status of the ac2 for a given mode */
unsigned short smrModes::usingAC2(unsigned short n){
  return *(usingac2+n);
}

/* public method, return the input port of the ac2 for a given mode */
unsigned short smrModes::ac2Input(unsigned short n){
  return *(ac2input+n);
}

/* public method, return the mux setting for AC2 */
unsigned short smrModes::ac2Mux(unsigned short n){
  return *(ac2mux+n);
}

/* public method, return the SSB1 setting for AC2 */
unsigned short smrModes::ac2SSB1(unsigned short n){
  return *(ac2ssb1+n);
}

/* public method, return the SSB2 setting for AC2 */
unsigned short smrModes::ac2SSB2(unsigned short n){
  return *(ac2ssb2+n);
}

/* public method, return the SSB3 setting for AC2 */
unsigned short smrModes::ac2SSB3(unsigned short n){
  return *(ac2ssb3+n);
}

/* public method, return the SSB4 setting for AC2 */
unsigned short smrModes::ac2SSB4(unsigned short n){
  return *(ac2ssb4+n);
}

/* public method, return the input cal status AC2 */
unsigned short smrModes::ac2SelfCal(unsigned short n){
  return *(selfcalac2+n);
}

/* public method, return attenuator setting 1 for AC2 */
unsigned short smrModes::ac2ATT1(unsigned short n){
  return *(ac2att1+n);
}

/* public method, return attenuator setting 2 for AC2 */
unsigned short smrModes::ac2ATT2(unsigned short n){
  return *(ac2att2+n);
}

/* public method, return attenuator setting 3 for AC2 */
unsigned short smrModes::ac2ATT3(unsigned short n){
  return *(ac2att3+n);
}

/* public method, return attenuator setting 4 for AC2 */
unsigned short smrModes::ac2ATT4(unsigned short n){
  return *(ac2att4+n);
}

/* public method, return the status of the filterbank for a given mode */
unsigned short smrModes::usingFilterbank(unsigned short n){
  return *(usingfilterbank+n);
}

/* public method, return the status of chop sync for a given mode */
unsigned short smrModes::chopSync(unsigned short n){
  return *(chopsync+n);
}

/* public method, return the status of 119 frequency switch sync */
unsigned short smrModes::F119Sync(unsigned short n){
  return *(f119sync+n);
}

/* public method, return the integration time for a given mode */
unsigned short smrModes::intTime(unsigned short n){
  return *(inttime+n);
}

/* public method, return the version number of the Radiometer Modes file */
unsigned short smrModes::versionNumber(){
  return modesVer;
}

/* public method, return the final commands */
char *smrModes::finalCmd(short n){
  if (*(discipline+n)==ASTRO) 
    return final_astro_cmd;
  
  return final_strat_cmd;
}

/* public method, return the file format */
short smrModes::fileFormat(){
  return fileformat;
}

/* private method, construct the Radiometer modes file name
 * used in method openModeFile
 */
char *smrModes::modeFilename(char *frootname, char *fextension){
  strstream *fname=new strstream;
  *fname << frootname
	 << dec << setw(2) << setprecision(2) << setfill('0')
	 << modesVer
	 << "."
	 << fextension
	 << ends;
  return fname->str();
}

/* private method, find the Radiometer modes file */
bool smrModes::openModeFile(){
  modesVer=99;

  char *frootname[]={"Radiometer_modes_v","modes"};
  short nNames=2;
  short nameCounter=0;
  char *fextension[]={"csv","skv"};
  short nExt=2;
  short extCounter=0;

  char *fname;
  while (modesVer >= 19){

    for (nameCounter=0;nameCounter<nNames;nameCounter++){
      for (extCounter=0;extCounter<nExt;extCounter++){
	fname=modeFilename(frootname[nameCounter],fextension[extCounter]);
	modeFile=new fstream;
	modeFile->open(fname,ios::nocreate | ios::in);
	if (!modeFile->fail()){
	  cout << "\nsuccessfully opened file: "
	       << fname;
	  return true;
	}
	delete modeFile;
      }
    }
    modesVer--;
  }
  cout << "\ncould not find Radiometer Modes file\n";
  return false;
}

/* private method, read the final commands found in text files */
bool smrModes::readFinalCmd(unsigned short _discipline){
  /* open file with extra commands. */
  fstream *cmdsFile=new fstream;
  if (_discipline==ASTRO)
    cmdsFile->open("astro.txt",ios::in | ios::nocreate);
  else
    cmdsFile->open("strat.txt",ios::in | ios::nocreate);

  /* on failure, return with warning message */
  if (cmdsFile->fail()) {
    cerr << "\nWARNING: no additional commands for ";
    if (_discipline==ASTRO){
      cerr << "astro modes:  astro.txt not found";
      final_astro_cmd="";
    }
    else {
      cerr << "strat modes:  strat.txt not found";
      final_strat_cmd="";
    }
    return false;
  }

  strstream *mkSeq = new strstream;
  char *buf=new char[132];
  while (!cmdsFile->eof()){
    cmdsFile->getline(buf,132);
    if (cmdsFile->eof())break;
    *mkSeq << buf << "\n";
  }
  *mkSeq << ends;
  cmdsFile->close();
  if (_discipline==ASTRO)
    final_astro_cmd=mkSeq->str();
  else
    final_strat_cmd=mkSeq->str();
  return true;
}

/* private method, go to first data line in the Radiometer Modes file */
bool smrModes::gotoStart(){
  fileformat=FILEFORMAT_ORIGINAL;
  char *row=new char[1024];
  modeFile->seekg(0,ios::end);
  cout << "\nfile is " << modeFile->tellg()  << " bytes";
  modeFile->seekg(0,ios::beg);
  while (!modeFile->eof()){
    modeFile->getline(row,1024,'\n');
    if (strncmp(row,"Receiver mode config",20)==0){
      char *pos=strstr(row,"chop sync");
      if (pos){
	fileformat++;
	pos=strstr(row,"mux");
	if (pos){
	  fileformat++;
	  pos=strstr(row,"self cal");
	  if (pos){
	    fileformat++;
	  }
	  else {
	    cout << "\nWARNING:  no attenuator settings found."
		 << "\n          using self calibration as default";
	  }
	}
	else {
	  cout << "\nWARNING:  no correlator settings found."
	       << "\n          using default settings for correlator modes";
	}
      }
      else {
	cout << "\nWARNING:  Radiometer Modes old format"
	     << "\n          using default settings"
	     << " for chop sync, 119 sync, and integration time";
      }

      /* before returning, determine what is the delimiter
       * comma or semi-colon */
      streampos firstline=modeFile->tellg();
      modeFile->getline(row,1024,'\n');
      modeFile->getline(row,1024,'\n');
      *delimiter=*row;
      modeFile->seekg(firstline,ios::beg);
      return true;
    }
  }

  cout << "\ncould not find start of mode data\n";
  return false;
}

/* private method, count the total number of modes
 * this should be called after first getting to the first line
 */
bool smrModes::countModes(){
  nmodes=0;
  streampos firstline=modeFile->tellg();
  short counter;
  char *row=new char[1024];
 
  while (!modeFile->eof()){

    /* read the first line and make sure it's valid */
    modeFile->getline(row,1024,'\n');
    if (strncmp(row,delimiter,1)==0) break;
    

    /* skip over the next four lines */
    for (counter=0;counter<4;counter++){
      modeFile->getline(row,1024,'\n');
      if (modeFile->eof())break;
    }

    /* if we've got this far, then a full mode is defined in the spreadsheet */
    nmodes++;
  }

  if (nmodes<=0){
    cout << "\nError:  no modes found!" << flush;
    modeFile->close();
    return false;
  }
  
  cout << "\nnumber of modes=" << nmodes << flush;
  modeFile->seekg(firstline,ios::beg);
  return true;
}

/* private method, reserve memory
 * this is quick and dirty for now
 * eventually, we should count the number of modes first
 * Calgary, Monday, 2001 June 04, 11:15:18
 *
 * Calgary-to-Toronto, Monday, 2001 June 11, 12:31:51
 * modes are now being counted
 */
bool smrModes::init(){
  if (nmodes<=0) return false;

  variation=new char[nmodes];
  name=new char* [nmodes];
  description=new char* [nmodes];
  discipline=new unsigned short[nmodes];
  obs_strategy=new unsigned short[nmodes];

  aosinput=new unsigned short[nmodes];
  ac1input=new unsigned short[nmodes];
  ac2input=new unsigned short[nmodes];

  aosmode=new unsigned short[nmodes];
  ac1mode=new unsigned short[nmodes];
  ac2mode=new unsigned short[nmodes];

  ac1ssb1=new unsigned short[nmodes];
  ac1ssb2=new unsigned short[nmodes];
  ac1ssb3=new unsigned short[nmodes];
  ac1ssb4=new unsigned short[nmodes];

  ac2ssb1=new unsigned short[nmodes];
  ac2ssb2=new unsigned short[nmodes];
  ac2ssb3=new unsigned short[nmodes];
  ac2ssb4=new unsigned short[nmodes];

  ac1mux=new unsigned short[nmodes];
  ac2mux=new unsigned short[nmodes];

  usingaos=new unsigned short[nmodes];
  usingac1=new unsigned short[nmodes];
  usingac2=new unsigned short[nmodes];
  usingfilterbank=new unsigned short[nmodes];

  selfcalaos=new unsigned short[nmodes];
  selfcalac1=new unsigned short[nmodes];
  selfcalac2=new unsigned short[nmodes];

  aosatt=new unsigned short[nmodes];

  ac1att1=new unsigned short[nmodes];
  ac1att2=new unsigned short[nmodes];
  ac1att3=new unsigned short[nmodes];
  ac1att4=new unsigned short[nmodes];

  ac2att1=new unsigned short[nmodes];
  ac2att2=new unsigned short[nmodes];
  ac2att3=new unsigned short[nmodes];
  ac2att4=new unsigned short[nmodes];

  chopsync=new unsigned short[nmodes];
  f119sync=new unsigned short[nmodes];
  inttime=new unsigned short[nmodes];

  return true;
}

/* private method, read the first line of a mode description
 * this contains the mode name, mode varation, Correlator 1 settings,
 * and the mode description.
 * return false, if the last mode has been read (indicated by an initial comma)
 */
bool smrModes::readFirstLine(short n, char *row){

  char *pos;
  short len;
  short i;

  /* if the first character in the row is a comma, this is not
   * the first line of an observation mode */
  if (strncmp(row,delimiter,1)==0)return false;

  /* read the first column entry */
  pos=strtok(row,delimiter);
  len=strlen(pos);

  /* the mode name and variation are in the first column entry */
  *(variation+n)=*(pos+len-1);
  *(name+n)=new char[len];
  for (i=0;i<len-1;i++) *(*(name+n)+i)=*(pos+i);
  *(*(name+n)+len-1)='\0';

  /* the discipline is determined by the first letter of the name */
  /* 'p' indicates astro mode, position switching */
  if (strncmp(*(name+n),"a",1)==0){
    *(discipline+n)=ASTRO;
    *(obs_strategy+n)=DICKE_SWITCHING;
  }
  else if (strncmp(*(name+n),"p",1)==0){
    *(discipline+n)=ASTRO;
    *(obs_strategy+n)=POSITION_SWITCHING;
  }
  else{
    *(discipline+n)=STRAT;
    *(obs_strategy+n)=DICKE_SWITCHING;
  }
  return true;
}

/* private method, extract the SSB settings
 * and mode description along the way
 */
bool smrModes::readSSB(short n, char *pos,
		       unsigned short *acmux,
		       unsigned short *ssb1,
		       unsigned short *ssb2,
		       unsigned short *ssb3,
		       unsigned short *ssb4){

  /* this is the chop cal position */
  pos=strtok(NULL,delimiter);
  

  /* this is the selected frontend (text) */
  pos=strtok(NULL,delimiter);

  /* string stream to hold the mode description */
  strstream mkDesc;

  /* this is the mode description
   * and here is where we have trouble 
   * because it may contain commas or semi-colons
   */
  pos=strtok(NULL,delimiter);
  if (pos) mkDesc << pos;

  /* this should be the mux setting
   * erroneous valid entries today are 0,10,64,84,127
   * see email sent to ufr and gf today, 2001Dec05
   *
   * Calgary, Thursday, 2001 December 06, 11:53:43
   * those values are not erroneous.  
   * We have all new correlator mux settings.
   * currently valid mux entries are 127,42,8,0,64,68,84,10
   *
   * we will search for the four valid SSB settings
   * and then assign the mux, all being well...
   */
  pos=strtok(NULL,delimiter);
  char *cmux=pos;

  /* this should be SSB1 setting */
  pos=strtok(NULL,delimiter);
  bool gotit=false;
  int len;
  while (pos && !gotit){

    /* the ssb setting must be 4 characters,
     * and must be an integer between 3400 and 4400
     */
    len=strlen(pos);
    if (len==4){
      len=atoi(pos);
      if (len>=3400 && len<=4400){
	gotit=true;
	break;
      }
    }

    /* if it's not the ssb setting, then the previous
     * is part of the description and we should keep going
     */
    mkDesc << delimiter << cmux;
    
    /* shift the stack (ie. the current token might be the mux setting) */
    cmux=pos;
    pos=strtok(NULL,delimiter);
  }

  if (!gotit){
    cout << "\nERROR:  could not read correlator modes\n";
    return false;
  }

  /* assign the mode description */
  mkDesc << ends;
  char *ttl=mkDesc.str();
  len=strlen(ttl)+1;
  *(description+n)=new char[len];
  for (int i=0;i<len-1;i++) *(*(description+n)+i)=*(ttl+i);
  *(*(description+n)+len-1)='\0';
#ifdef DEBUG
  cerr << "\ncorrelator mode description:  " << *(description+n);
#endif

  /* assign the mux setting */
  *(acmux+n)=atoi(cmux);

  /* assign the SSB1 */
  *(ssb1+n)=atoi(pos);

  /* this should be the SSB2 */
  pos=strtok(NULL,delimiter);
  *(ssb2+n)=atoi(pos);

  /* this should be the SSB3 */
  pos=strtok(NULL,delimiter);
  *(ssb3+n)=atoi(pos);

  /* this should be the SSB4 */
  pos=strtok(NULL,delimiter);
  *(ssb4+n)=atoi(pos);

  return true;
}

/* private method, create an error message for use in method readAtt */
char *smrModes::readAttErr(short n, unsigned short whichBck){
  strstream mkErr;
  mkErr << "\n         defaulting to self-calibration for mode "
	<< modeName(n) << modeVariation(n)
	<< " on ";
  switch(whichBck){
  case AC1: mkErr << "AC1";
    break;
  case AC2: mkErr << "AC2";
    break;
  case AOS: mkErr << "AOS";
    break;
  case FB: mkErr << "filterbank";
    break;
  default: mkErr << "unidentified backend!!!";
  }
  mkErr << ends;
  return mkErr.str();
}

/* private method, read the manual attenuator setting for the AOS
 * this is similar to the correlator readSSB method because 
 * we have to get through the mode description
 */
bool smrModes::readAOSatt(short n, char *pos){
  /* initialize attenuator setting to zero */
  *(aosatt+n)=0;

  /* string stream to hold the mode description */
  strstream mkDesc;

  /* this is the chop cal position */
  pos=strtok(NULL,delimiter);
  
  /* this is the selected frontend (text) */
  pos=strtok(NULL,delimiter);

  /* this is the mode description
   * and here is where we have trouble 
   * because it may contain commas or semi-colons
   */
  pos=strtok(NULL,delimiter);
  if (pos) 
    mkDesc << pos;
  else {
    cout << "\nWARNING: no description nor self-cal column detected for AOS"
	 << readAttErr(n,AOS);
    *(selfcalaos+n)=ON;
    return true;
  }

  /* the next column should be self-cal yes or no */
  pos=strtok(NULL,delimiter);
  if(!pos){
    cout << "\nWARNING: no self-cal column detected for AOS"
	 << readAttErr(n,AOS);
    *(selfcalaos+n)=ON;
    return true;
  }

  /* we will search for the valid attenuator setting
   * if there is nothing left in the spreadsheet
   * then exit and default to self-cal
   */
  bool gotit=false;
  int len,val;
  while (pos && !gotit){


    /* the on/off self-cal entry must be 1 character,
     * if it's not the self-cal entry,
     * then it is part of the description and we should keep going
     */
    len=strlen(pos);
    if (len!=1){
      mkDesc << delimiter << pos;
      pos=strtok(NULL,delimiter);
      continue;
    }

    /* the on/off self-cal entry  must be an integer either 0 or 1 */
    val=atoi(pos);
    if (val!=ON && val!=OFF){
      mkDesc << delimiter << pos;
      pos=strtok(NULL,delimiter);
      continue;
    }
    *(selfcalaos+n)=val;
  
    /* the following column should be a valid attenuator setting
     * but only post an error message of self cal was specified as OFF
     * and no attenuator setting was specified
     */
    pos=strtok(NULL,delimiter);
    if(!pos){
      if (val==OFF)
	cerr << "\nERROR:   no attenuator setting specified"
	     << readAttErr(n,AOS);
      break;
    }

    /* the attenuator setting must be only 1 or 2 characters long */
    len=strlen(pos);
    if (len!=1 && len!=2){
      mkDesc << delimiter << pos;
      pos=strtok(NULL,delimiter);
      continue;
    }

    /* check that the value is in range */
    val=atoi(pos);
    if (val<AOS_MIN_ATT || val>AOS_MAX_ATT){
      mkDesc << delimiter << pos;
      pos=strtok(NULL,delimiter);
      continue;
    }

    /* if we got this far, we've got it 
     * (but it could still be an unlucky coincidence ... 
     */
    *(aosatt+n)=val;
    gotit=true;

  }

  /* if we never found a valid attenuator setting
   * then quietly default to self cal
   * a warning message must already have been output */
  if (!gotit) *(selfcalaos+n)=ON;

  /* finish off the mode description stream */
  mkDesc << ends;
#ifdef DEBUG
  cerr << "\nAOS mode description:  " << mkDesc.str();
#endif

  return true;
}


/* private method, read the manual attenuator settings */
bool smrModes::readAtt(short n, char *pos,
		       unsigned short whichBck,
		       unsigned short *selfcal,
		       unsigned short *att1,
		       unsigned short *att2,
		       unsigned short *att3,
		       unsigned short *att4){

  /* if it's the AOS, use the readAOSatt method
   * to get past the mode description found on that row
   */
  if(whichBck==AOS)return readAOSatt(n,pos);

  /* there is AOS specific stuff below, maybe it should be deleted... */

  /* initialize attenuator settings to zero */
  *(att1+n)=0;
  *(att2+n)=0;
  *(att3+n)=0;
  *(att4+n)=0;

  /* this is the self-cal yes/no value
   * if we are doing self-cal
   * then there's no need to try and read the attenuator settings */
  pos=strtok(NULL,delimiter);

  /* check if we've got a non null entry.
   * otherwise, default to self-cal and exit */
  if (!pos){
    *(selfcal+n)=ON;
    return true;
  }

  /* read selfcal ON/OFF
   * if self cal is ON, exit immediately */
  short chk=atoi(pos);
  if (chk!=OFF){
    *(selfcal+n)=ON;
    return true;
  }
  *(selfcal+n)=OFF;

  /* min/max valid attenuator levels 
   * note for the correlators, high values is minimum attenuation!
   */
  short minAtt;
  short maxAtt;
  if (whichBck==AOS){
    minAtt=AOS_MIN_ATT;
    maxAtt=AOS_MAX_ATT;
  }
  else{
    minAtt=AC_MAX_ATT;
    maxAtt=AC_MIN_ATT;
  }

  /* read the four attenuator settings
   * exit after the first if it's the AOS
   * exit with self-cal=ON for any non-valid entry */

  for (short i=0;i<4;i++){

    pos=strtok(NULL,delimiter);
    if (!pos){
      cout << "\nWARNING: null entry for attenuator setting."
	   << readAttErr(n,whichBck)
	   << endl;
      *(selfcal+n)=ON;
      return true;
    }

    /* read the attenuator level, exit if it's out of range */
    chk=atoi(pos);
    if ((chk<minAtt) || (chk>maxAtt)){
      cout << "\nWARNING: invalid attenuator setting: "
	   << dec << chk
	   << " is out of range "
	   << minAtt << " to " << maxAtt
	   << readAttErr(n,whichBck)
	   << endl;
      *(selfcal+n)=ON;
      return true;
    }

    /* assign the attenuator level */
#ifdef DEBUG
    cerr << "\nassigning attenuator "
	 << i
	 << " = " << chk
	 << " for backend " << whichBck
	 << " for mode " << modeName(n) << modeVariation(n);
#endif
    switch (i){
    case 0: *(att1+n)=(unsigned short) chk;
      break;
    case 1: *(att2+n)=(unsigned short) chk;
      break;
    case 2: *(att3+n)=(unsigned short) chk;
      break;
    case 3: *(att4+n)=(unsigned short) chk;
      break;
    default: cout << "\nERROR:  attenuator not assigned";
    }

    /* if it's the AOS, exit now */
    if (whichBck==AOS) return true;

  }

  return true;
}

/* private method, read a line in the Radiometer Modes file
 * and extract the backend parameters */
bool smrModes::readLine(short n, char *row){
  char *pos;
  unsigned short *usingBck; //holder for backend on/off parameter
  unsigned short *input; //holder for backend input parameter
  unsigned short *mode; //holder for backend mode parameter
  unsigned short *ssb1; //holder for correlator SSB settings
  unsigned short *ssb2; //holder for correlator SSB settings
  unsigned short *ssb3; //holder for correlator SSB settings
  unsigned short *ssb4; //holder for correlator SSB settings
  unsigned short *acmux; //holder for correlator SSB settings
  unsigned short *selfcal;//holder for backend self cal on/off
  unsigned short *att1;//holder for manual input attenuator setting
  unsigned short *att2;//holder for manual input attenuator setting
  unsigned short *att3;//holder for manual input attenuator setting
  unsigned short *att4;//holder for manual input attenuator setting
  unsigned short whichBck;//identity of current backend

  /* backend identity is in column I 
   * but we'll just search until we find it
   * maybe this will be more robust to changes in Radiometer_modes.xls
   */
  pos=strtok(row,delimiter);
  while (pos) {
    if (strncmp(pos,"Corr F1A",8)==0){
      whichBck=AC1;
      usingBck=usingac1;
      input=ac1input;
      mode=ac1mode;
      ssb1=ac1ssb1;
      ssb2=ac1ssb2;
      ssb3=ac1ssb3;
      ssb4=ac1ssb4;
      acmux=ac1mux;
      selfcal=selfcalac1;
      att1=ac1att1;
      att2=ac1att2;
      att3=ac1att3;
      att4=ac1att4;
      break;
    }
  
    if (strncmp(pos,"Corr F2B",8)==0){
      whichBck=AC2;
      usingBck=usingac2;
      input=ac2input;
      mode=ac2mode;
      ssb1=ac2ssb1;
      ssb2=ac2ssb2;
      ssb3=ac2ssb3;
      ssb4=ac2ssb4;
      acmux=ac2mux;
      selfcal=selfcalac2;
      att1=ac2att1;
      att2=ac2att2;
      att3=ac2att3;
      att4=ac2att4;
      break;
    }

    if (strncmp(pos,"AOS",3)==0){
      whichBck=AOS;
      usingBck=usingaos;
      input=aosinput;
      mode=aosmode;
      selfcal=selfcalaos;
      att1=aosatt;
      break;
    }

    if (strcmp(pos,"Filterbank")==0){
      whichBck=FB;
      usingBck=usingfilterbank;
      break;
    }

    pos=strtok(NULL,delimiter);
  }

  if (!usingBck){
    cout << "\ncould not find backend info in modes file";
    return false;
  }

  /* backend is the following three columns
   * in columns J,K,L (on/off, input, mode) 
   */
  pos=strtok(NULL,delimiter);
  if (!pos) return false;
  *(usingBck+n)=atoi(pos);

  /* if the backend is not in use for this mode
   * then there is nothing more to read 
   * (including the description, which is not used anyway)
   */
  if (*(usingBck+n)==OFF) return true;

  /* if it's the filterbank, return now */
  if (whichBck==FB) return true;

  /* input channel */
  pos=strtok(NULL,delimiter);
  if (!pos) return false;
  *(input+n)=atoi(pos);

  /* backend mode */
  pos=strtok(NULL,delimiter);
  if (!pos) return false;
  *(mode+n)=atoi(pos);

  /* if it's the old format of the Radiometer Modes file,
   * then assign default values
   */
  if (fileformat==FILEFORMAT_ORIGINAL){
    *(chopsync+n)=ON;
    *(f119sync+n)=OFF;
    if (Discipline(n)==STRAT)
      *(inttime+n)=4;
    else if (strncmp(modeName(n),"a12",3)==0)
      *(inttime+n)=5;
    else
      *(inttime+n)=10;
    return true;
  }

  /* if it's a later format of Radiometer Modes,
   * then we can read chop sync, 119 sync, and integration time
   */
  pos=strtok(NULL,delimiter);
  if (!pos) return false;
  *(chopsync+n)=atoi(pos);
  
  pos=strtok(NULL,delimiter);
  if (!pos) return false;
  *(f119sync+n)=atoi(pos);
  if(*(f119sync+n)==ON) *(obs_strategy+n)=F119_SWITCHING;

  pos=strtok(NULL,delimiter);
  if (!pos) return false;
  *(inttime+n)=atoi(pos);

  /* if we don't have correlator settings in the file
   * then don't try to read them
   */
  if (fileformat<FILEFORMAT_WITH_MUX_AND_SSBS) return true;


  /* if it's the file format that contains correlator settings
   * then we can read the SSB frequencies for each correlator
   * but not if the backend is the AOS
   */
  if (whichBck!=AOS) readSSB(n,pos,acmux,ssb1,ssb2,ssb3,ssb4);

  /* if it's the file format with manual attenuator settings
   * then we can read them.  The default is to do a self calibration
   */
  if(fileformat<FILEFORMAT_WITH_ATT){
    *(selfcal+n)=ON;
    return true;
  }

  return readAtt(n,pos,whichBck,selfcal,att1,att2,att3,att4);
}

/* public method, read the mode description file Radiometer_modes_vN.csv */
bool smrModes::readModes(){

  if (!openModeFile())return false;

  if (!gotoStart())return false;

  /* count modes: not implemented yet 
   * Calgary, Monday, 2001 June 04, 11:18:12
   * implemented: Calgary-to-Toronto, Monday, 2001 June 11, 12:17:09
   */
  if (!countModes())return false;

  /* reserve memory */
  if (!init())return false;

  /* read any extra commands that can be found in a text file */
  readFinalCmd(ASTRO);
  readFinalCmd(STRAT);

  /* create the char array to hold the row of spreadsheet data */
  char *row;
  char *cdum;

  /* a counter for reading backend data (counts to 4) */
  int counter;
  short modeCounter;

  for (modeCounter=0;modeCounter<nmodes;modeCounter++){

    /* read the first line of a mode description
     * if it's not valid, then all modes have been read
     * and we can return
     */
    row=new char[1024];
    cdum=new char[1024];
    modeFile->getline(row,1024,'\n');

    /* we have to copy row because it gets destroyed by the tokening */
    int len=strlen(row);
    for (counter=0;counter<len;counter++) *(cdum+counter)=*(row+counter);
    *(cdum+len)='\0';

    /* read the first line */
    if (!readFirstLine(modeCounter,row))break;
    row=cdum;
    /* read the backend data */
    for (counter=0;counter<4;counter++){
      if (!readLine(modeCounter,row)){
	cout << "\nsmrModes::readModes - failed readLine";
	return false;
      }
      if (modeFile->eof()){
	cout << "\nsmrModes::readModes - premature end of file";
	return false;
      }
      row=new char[1024];
      modeFile->getline(row,1024,'\n');
    }

  }
  return true;
}

