/* $Id: smrModes.h for Odin
 * $auth: Steve Torchinsky
 * $created: Calgary, Sunday, 2001 May 20, 20:41:15
 *
 * This is the part of the cpp version of mkbckseq
 * mkbckseq is a Bash script employing sed and gawk which
 * uses templates to create backend command scripts
 *
 * The cpp version will do the same job, and will also read
 * the mode configurations from text files exported from the
 * Radiometer_modes.xls excel spreadsheet by Urban Frisk
 *
 * 
 * The class template for reading and storing the radiometer modes
 * is defined here.
 * 
 * the naming convention in this class is the following:
 *    names with all lower case are private variables
 *    names with mixed upper and lower case are methods
 *
 * changes: 
 *  Calgary-to-Toronto, Monday, 2001 June 11, 12:17:09
 *  adding mode counting method countModes
 *
 *  Calgary, Thursday, 2001 June 14, 19:58:30
 *  adding flag for new Radiometer_modes_vNN.xls format
 *
 *  Calgary, Friday, 2001 August 03, 09:45:08
 *  final commands read from text file, moved to smrModes from bckCmd
 *    new methods:  finalCmd, readFinalCmd;
 *    new private variables: final_astro_cmd, final_strat_cmd
 *
 *  Calgary, Monday, 2002 February 04, 15:26:42
 *  v2.4.4
 *  adding methods and variables for manual attenuator settings
 *
 *  Calgary, Friday, 2002 February 22, 15:01:59
 *  new method readSSB
 *
 *  Calgary, Monday, 2002 February 25, 10:13:43
 *  new method, readAtt
 *
 *  Calgary, Monday, 2002 February 25, 15:32:38
 *  new method readAttErr
 *
 *  Calgary, Monday, 2002 February 25, 22:37:15
 *  new method readAOSatt to get past the mode description
 *
 *  St-Bruno, Thursday, 2002 October 10, 09:01:25
 *  new variable and methods for Observing Strategy  
 */

#ifndef SMRMODES_H
#define SMRMODES_H

#define DELIMITER ","

/* file formats */
#define FILEFORMAT_ORIGINAL 0
#define FILEFORMAT_WITH_SYNC_AND_INTTIME 1
#define FILEFORMAT_WITH_MUX_AND_SSBS 2
#define FILEFORMAT_WITH_ATT 3

#include <cstdlib>
#include <fstream>

#include "../OdinTM.h"
#include "OdinCmd.h"

enum spreadsheetColumn {A=1,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z};
			
class smrModes {

 public:
  smrModes();
  ~smrModes();

  bool readModes();
  unsigned short versionNumber();
  short nModes();
  char *modeDescription(unsigned short);
  char *modeName(unsigned short);
  char modeVariation(unsigned short);
  unsigned short Discipline(unsigned short);
  unsigned short ObservingStrategy(unsigned short);

  unsigned short aosInput(unsigned short);
  unsigned short ac1Input(unsigned short);
  unsigned short ac2Input(unsigned short);

  unsigned short aosMode(unsigned short);
  unsigned short ac1Mode(unsigned short);
  unsigned short ac2Mode(unsigned short);

  unsigned short ac1Mux(unsigned short);
  unsigned short ac2Mux(unsigned short);

  unsigned short ac1SSB1(unsigned short);
  unsigned short ac1SSB2(unsigned short);
  unsigned short ac1SSB3(unsigned short);
  unsigned short ac1SSB4(unsigned short);

  unsigned short ac2SSB1(unsigned short);
  unsigned short ac2SSB2(unsigned short);
  unsigned short ac2SSB3(unsigned short);
  unsigned short ac2SSB4(unsigned short);

  unsigned short usingAOS(unsigned short);
  unsigned short usingAC1(unsigned short);
  unsigned short usingAC2(unsigned short);
  unsigned short usingFilterbank(unsigned short);

  unsigned short aosSelfCal(unsigned short);
  unsigned short ac1SelfCal(unsigned short);
  unsigned short ac2SelfCal(unsigned short);

  unsigned short aosATT(unsigned short);

  unsigned short ac1ATT1(unsigned short);
  unsigned short ac1ATT2(unsigned short);
  unsigned short ac1ATT3(unsigned short);
  unsigned short ac1ATT4(unsigned short);

  unsigned short ac2ATT1(unsigned short);
  unsigned short ac2ATT2(unsigned short);
  unsigned short ac2ATT3(unsigned short);
  unsigned short ac2ATT4(unsigned short);

  unsigned short chopSync(unsigned short);
  unsigned short F119Sync(unsigned short);
  unsigned short intTime(unsigned short);

  char *finalCmd(short);
  short fileFormat();

 private:
  fstream *modeFile;
  char *modeFilename(char *, char *);
  bool openModeFile();
  bool gotoStart();
  bool countModes();
  bool readFirstLine(short, char *);
  bool readSSB(short, char *,
	       unsigned short *,
	       unsigned short *,
	       unsigned short *,
	       unsigned short *,
	       unsigned short *);
  bool readAtt(short, char *,
	       unsigned short,
	       unsigned short *,
	       unsigned short *,
	       unsigned short *,
	       unsigned short *,
	       unsigned short *);
  bool readAOSatt(short,char *);
  char *readAttErr(short, unsigned short);
  bool readLine(short, char *);
  bool init();
  bool readFinalCmd(unsigned short);

  unsigned short modesVer;

  short nmodes;
  char *variation;
  char **name;
  char **description;
  unsigned short *discipline;
  unsigned short *obs_strategy;

  unsigned short *aosinput;
  unsigned short *ac1input;
  unsigned short *ac2input;

  unsigned short *aosmode;
  unsigned short *ac1mode;
  unsigned short *ac2mode;

  unsigned short *usingaos;
  unsigned short *usingac1;
  unsigned short *usingac2;
  unsigned short *usingfilterbank;

  unsigned short *selfcalaos;
  unsigned short *selfcalac1;
  unsigned short *selfcalac2;

  unsigned short *aosatt;

  unsigned short *ac1att1;
  unsigned short *ac1att2;
  unsigned short *ac1att3;
  unsigned short *ac1att4;

  unsigned short *ac2att1;
  unsigned short *ac2att2;
  unsigned short *ac2att3;
  unsigned short *ac2att4;

  unsigned short *ac1ssb1;
  unsigned short *ac1ssb2;
  unsigned short *ac1ssb3;
  unsigned short *ac1ssb4;

  unsigned short *ac2ssb1;
  unsigned short *ac2ssb2;
  unsigned short *ac2ssb3;
  unsigned short *ac2ssb4;

  unsigned short *ac1mux;
  unsigned short *ac2mux;

  unsigned short *chopsync;
  unsigned short *f119sync;
  unsigned short *inttime;
  short fileformat;

  char *final_astro_cmd;
  char *final_strat_cmd;

  char *delimiter;

};

#endif

